
/*

  $Id: filelist.c,v 1.12 2003/08/08 22:48:23 cramer Exp $
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <sys/types.h>
#include <dirent.h>
#include <pthread.h>

#include "include/filelist.h"
#include "include/plotdive.h"
#include "include/readudcf.h"
#include "include/defines.h"
#include "support.h"

#define FILELEN (50)
#define MAX_NUMFILES (1000)

static int          mycmp (void *s, void *t);

struct DiveProfile *CurrentDiveProfile = NULL;
extern int          RefreshList;
guint               idleFileList;

GtkListStore       *ListStore;
pthread_mutex_t     listmutex = PTHREAD_MUTEX_INITIALIZER;

void buildFileList (GtkWidget * widget, char *dir, int first)
{
  GtkWidget          *diveList;
  GtkTreeIter         iter;
  DIR                *filelist;
  int                 i = 0, numfiles = 0;
  struct dirent      *direntry;
  GtkTreeViewColumn  *column;
  GtkCellRenderer    *renderer;
  GtkTreeSelection   *select;
  unsigned int        divenum;
  char                divedate[11], divetime[5];
  char                filenames[MAX_NUMFILES][FILELEN];

  pthread_mutex_lock (&listmutex);

  /* Get container widgets */
  diveList = lookup_widget (widget, "diveList");

  if (first) {
    ListStore =
        gtk_list_store_new (N_COLUMNS, G_TYPE_LONG, G_TYPE_STRING,
                            G_TYPE_STRING, -1);
  } else {
    gtk_list_store_clear (ListStore);
  }

  filelist = opendir (dir);
  if (filelist != NULL) {
    while ((direntry = readdir (filelist)) != NULL) {
      if (strstr (direntry->d_name, ".udcf") == NULL ||
          strstr (direntry->d_name, "~") != NULL) {
      } else {
        strcpy ((char *) filenames[numfiles++], direntry->d_name);
      }
    }

    printf ("numfiles = %d\n", numfiles);

    qsort (filenames, numfiles, FILELEN, (void *) mycmp);

    for (i = 0; i < numfiles; i++) {
      sscanf ((char *) filenames[i], "%d-%10s_%4s.udcf", &divenum,
              divedate, divetime);

      gtk_list_store_append (ListStore, &iter);
      gtk_list_store_set (ListStore, &iter, DIVENUM_COLUMN, divenum, -1);
      gtk_list_store_set (ListStore, &iter, DIVEDATE_COLUMN, &divedate, -1);
      gtk_list_store_set (ListStore, &iter, DIVETIME_COLUMN, &divetime, -1);
    }

    if (first) {
      gtk_tree_view_set_model (GTK_TREE_VIEW (diveList),
                               GTK_TREE_MODEL (ListStore));

      renderer = gtk_cell_renderer_text_new ();
      column = gtk_tree_view_column_new_with_attributes ("Dive", renderer,
                                                         "text",
                                                         DIVENUM_COLUMN,
                                                         NULL);
      gtk_tree_view_append_column (GTK_TREE_VIEW (diveList), column);

      renderer = gtk_cell_renderer_text_new ();
      column = gtk_tree_view_column_new_with_attributes ("Date", renderer,
                                                         "text",
                                                         DIVEDATE_COLUMN,
                                                         NULL);
      gtk_tree_view_append_column (GTK_TREE_VIEW (diveList), column);

      renderer = gtk_cell_renderer_text_new ();
      column = gtk_tree_view_column_new_with_attributes ("Time", renderer,
                                                         "text",
                                                         DIVETIME_COLUMN,
                                                         NULL);
      gtk_tree_view_append_column (GTK_TREE_VIEW (diveList), column);

      select = gtk_tree_view_get_selection (GTK_TREE_VIEW (diveList));
      g_signal_connect_after (G_OBJECT (select), "changed",
                              G_CALLBACK (on_ListStore_changed), widget);
    }
    closedir (filelist);
  } else {
    switch (errno) {
    case EACCES:
      printf ("Access denied!\n");
      break;
    case EMFILE:
      printf ("To many filedescriptors open\n");
      break;
    case ENFILE:
      printf ("To many open files on system\n");
      break;
    case ENOENT:
      printf ("Direcory does not exist\n");
      break;
    case ENOMEM:
      printf ("Insufficient memory to complete the operation");
      break;
    case ENOTDIR:
      printf ("%s is not a directory\n", dir);
      break;
    }
  }
  pthread_mutex_unlock (&listmutex);
}

void on_ListStore_changed (GtkTreeSelection * select, gpointer user_data)
{
  GtkTreeIter         iter;
  GtkTreeModel       *model;
  int                 divenum;
  gchar              *divedate;
  gchar              *divetime;
	gchar              filename[255];
  
  if (gtk_tree_selection_get_selected (select, &model, &iter)) {
    gtk_tree_model_get (model, &iter, DIVENUM_COLUMN, &divenum, -1);
    gtk_tree_model_get (model, &iter, DIVEDATE_COLUMN, &divedate, -1);
    gtk_tree_model_get (model, &iter, DIVETIME_COLUMN, &divetime, -1);

    sprintf (filename, "%s/%s/%d-%s_%s.udcf", getenv ("HOME"), ALADINHOME,
             divenum, divedate, divetime);
    CurrentDiveProfile = readudcf (filename);

		/*
		if (CurrentDiveProfile->filename != NULL) {
		  free(CurrentDiveProfile->filename);
		}
		CurrentDiveProfile->filename = malloc(strlen(filename)+1);
		strcpy(CurrentDiveProfile->filename, filename);
		*/
		
    plotDive (user_data, PLOT_ALL);
  }
}

static int mycmp (void *s, void *t)
{
  int                 a, b;

  sscanf (s, "%d-", &a);
  sscanf (t, "%d-", &b);

  return (a - b);
}
