
/*

  $Id: plotdive.c,v 1.14 2003/06/08 10:44:18 cramer Exp $
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <math.h>

#include "support.h"
#include "include/plotdive.h"
#include "include/readudcf.h"

extern struct DiveProfile *CurrentDiveProfile;
static gboolean     CreateColor (GdkColor * Col, int Red, int Green,
                                 int Blue);

gint plotDive (GtkWidget * widget, int what)
{

  GtkWidget          *DiveNum;
  GtkWidget          *DiveTime;
  GtkWidget          *MaxDepth;
  GtkWidget          *SurfaceInterval;
  GtkWidget          *graphLabel;
  GtkWidget          *DeltaP;
  GtkWidget          *diveProfile;
  GtkWidget          *BreathVolume;
  GtkWidget          *AvgDepth;
  gchar               temp[255];
  int                 width, height;
  float               xinc, yinc;
  int                 xmul, ymul;
  int                 x, y, oldx, oldy;
  int                 i;
  float               minutes;
  PangoContext       *pc;
  PangoLayout        *pl;
  PangoFontDescription *pfd;
  GdkGC              *gc;
  GdkColor            color;

  if (what & PLOT_LABELS) {
    DiveNum = lookup_widget (widget, "DiveNum");
    DiveTime = lookup_widget (widget, "DiveTime");
    MaxDepth = lookup_widget (widget, "MaxDepth");
    SurfaceInterval = lookup_widget (widget, "SurfaceInterval");
    DeltaP = lookup_widget (widget, "DeltaP");
    BreathVolume = lookup_widget (widget, "BreathVolume");
    AvgDepth = lookup_widget (widget, "AvgDepth");

    graphLabel = lookup_widget (widget, "graphLabel");
    sprintf (temp, "<span size='large'>%04d</span>",
             CurrentDiveProfile->DiveNum);
    gtk_label_set_markup (GTK_LABEL (DiveNum), temp);

    sprintf (temp,
             "<span size='x-small'>Start time:</span> <span size='small'>%02d.%02d.%04d %02d:%02d %s</span>",
             CurrentDiveProfile->day, CurrentDiveProfile->month,
             CurrentDiveProfile->year, CurrentDiveProfile->hour,
             CurrentDiveProfile->minute, CurrentDiveProfile->timezone);
    gtk_label_set_markup (GTK_LABEL (DiveTime), temp);

    sprintf (temp,
             "<span size='x-small'>Max depth:</span> <span size='small'>%.2f m</span>",
             CurrentDiveProfile->maxDepth);
    gtk_label_set_markup (GTK_LABEL (MaxDepth), temp);

    sprintf (temp,
             "<span size='x-small'>Delta P:</span> <span size='small'>%u Bar</span>",
             CurrentDiveProfile->pDelta);
    gtk_label_set_markup (GTK_LABEL (DeltaP), temp);

    if (CurrentDiveProfile->SurfaceIntervall > 0) {
      sprintf (temp,
               "<span size='x-small'>Surface Intervall:</span> <span size='small'>%d:%02d</span>",
               CurrentDiveProfile->SurfaceIntervall / 60,
               CurrentDiveProfile->SurfaceIntervall % 60);
    } else {
      sprintf (temp,
               "<span size='x-small'>Surface Intervall:</span> <span size='small'>Infinit</span>");
    }
    gtk_label_set_markup (GTK_LABEL (SurfaceInterval), temp);

    sprintf (temp,
             "<span size='x-small'>Avg Depth:</span> <span size='small'>%.2f m</span>",
             CurrentDiveProfile->avgDepth);
    gtk_label_set_markup (GTK_LABEL (AvgDepth), temp);

    sprintf (temp,
             "<span size='x-small'>Breath Volume:</span> <span size='small'>%.2f l/min</span>",
             CurrentDiveProfile->bVol);
    gtk_label_set_markup (GTK_LABEL (BreathVolume), temp);

  }

  /* Get widget */
  if (what & (PLOT_CLEAR | PLOT_GRAPH)) {
    diveProfile = lookup_widget (widget, "diveProfile");
  }

  /* Clear drawing area */
  if (what & PLOT_CLEAR) {
    gdk_window_clear (diveProfile->window);
  }

  /* Now plot the stuff */
  if (what & PLOT_GRAPH) {
#define XOFF (20)
#define YOFF (20)

    width = diveProfile->allocation.width;
    height = diveProfile->allocation.height;

    xinc = (width - XOFF - 5) / (float) CurrentDiveProfile->numsamples * 1.0;
    yinc = (height - YOFF - 5) / CurrentDiveProfile->maxDepth * 1.0;

    gdk_draw_line (diveProfile->window,
                   diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)], 20,
                   20, width - 5, 20);
    gdk_draw_line (diveProfile->window,
                   diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)], 20,
                   20, 20, height - 5);

    oldx = XOFF;
    oldy = YOFF;

    /* initialize font stuff */
    pc = gtk_widget_create_pango_context (diveProfile);
    pl = pango_layout_new (pc);
    pfd = pango_font_description_from_string ("Helvetica,Medium 10");
    pango_layout_set_font_description (pl, pfd);

    gc = gdk_gc_new (widget->style);
    CreateColor (&color, 0xAAAA, 0xAAAA, 0xAAAA);
    gdk_gc_set_foreground (gc, &color);

    /* draw the time scale */
    minutes =
        CurrentDiveProfile->delta * CurrentDiveProfile->numsamples / 60.0;
    // printf("minutes: %.2f\n", minutes);

    xmul = (int) (minutes * 45 / ((float) width - XOFF - 5.0));

    if (xmul < 1)
      xmul = 1;
    for (i = 0; i <= minutes; i += xmul) {
      x = (int) ((i * 60 * xinc / CurrentDiveProfile->delta) + XOFF);
      if (i > 0)
        gdk_draw_line (diveProfile->window, gc, x, YOFF + 5, x, height - 5);

      gdk_draw_line (diveProfile->window,
                     diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)], x,
                     YOFF + 5, x, YOFF - 5);
      sprintf (temp, "<span size='x-small'>%d</span>", i);
      pango_layout_set_markup (pl, temp, -1);
      gdk_draw_layout (diveProfile->window,
                       diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)],
                       x - 4, 1, pl);
    }

    /* draw the depth scale */
    ymul =
        (int) (CurrentDiveProfile->maxDepth * 45 /
               ((float) height - YOFF - 5.0));

    if (ymul < 1)
      ymul = 1;
    for (i = 0; i <= CurrentDiveProfile->maxDepth; i += ymul) {
      y = (i * yinc) + YOFF;
      if (i > 0)
        gdk_draw_line (diveProfile->window, gc, XOFF + 5, y, width - 5, y);
      gdk_draw_line (diveProfile->window,
                     diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)],
                     XOFF - 5, y, XOFF + 5, y);
      sprintf (temp, "<span size='x-small'>%d</span>", i);
      pango_layout_set_markup (pl, temp, -1);
      gdk_draw_layout (diveProfile->window,
                       diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)],
                       1, y - 7, pl);
    }

		/* Draw the avg depth line */
		CreateColor (&color, 0xFFFF, 0xFFFF, 0x0000);
    gdk_gc_set_foreground (gc, &color);
		y = (int)(CurrentDiveProfile->avgDepth * yinc + YOFF);
		gdk_draw_line (diveProfile->window,
                     gc,
                     XOFF, y, width -5, y);
		
    /* now plot the profile */
    for (i = 0; i < CurrentDiveProfile->numsamples; i++) {
      x = (int) ((i) * xinc + XOFF);
      y = (int) (CurrentDiveProfile->profile[i] * yinc + YOFF);
      gdk_draw_line (diveProfile->window,
                     diveProfile->style->fg_gc[GTK_WIDGET_STATE (widget)],
                     oldx, oldy, x, y);
      oldx = x;
      oldy = y;
    }
  }

  return FALSE;
}

static gboolean CreateColor (GdkColor * Col, int Red, int Green, int Blue)
{
  GdkColormap        *cmap;

  cmap = gdk_colormap_get_system ();
  Col->red = Red;
  Col->green = Green;
  Col->blue = Blue;
  if (!gdk_colormap_alloc_color (cmap, Col, FALSE, TRUE)) {
    g_error ("couldn't allocate color");
    return FALSE;
  }
  return TRUE;
}
