
/*

  $Id: readudcf.c,v 1.12 2003/08/08 22:48:23 cramer Exp $
*/

#include <stdio.h>
#include <expat.h>
#include <string.h>
#include "include/readudcf.h"

/* Prototypes for static functions */
static void         charHandler (struct XMLHeap *userData, const XML_Char * s,
                                 int len);
static void         endElementHandler (struct XMLHeap *userData,
                                       const char *name);
static void         startElementHandler (struct XMLHeap *userData,
                                         const char *name, const char **atts);
static void         fillvalues (struct XMLHeap *userData);


static void
startElementHandler (struct XMLHeap *userData, const char *name,
                     const char **atts)
{
  userData->level++;
  strncpy (userData->tags[userData->level], name, strlen (name) + 1);
  // printf("We are now on level %d and the tag we've seen is \"%s\"\n",
  //   userData->level, userData->tags[userData->level]);
}

static void endElementHandler (struct XMLHeap *userData, const char *name)
{
  if (userData->done != 1) {
    if (!strncasecmp (userData->tags[1], "REPGROUP", 8) &&
        !strncasecmp (name, "REPGROUP", 8)) {

      userData->done = 1;
      userData->diveprofile->avgDepth =
          userData->diveprofile->avgDepth / userData->diveprofile->numsamples;
      userData->diveprofile->bVol =
          (userData->diveprofile->pDelta * userData->diveprofile->tanksize) /
          (((userData->diveprofile->avgDepth / 10) +
            1) * ((userData->diveprofile->numsamples *
                   userData->diveprofile->delta) / 60));
      // printf("We're done :-)\n");
    } else {
      fillvalues (userData);
    }
  }
  userData->level--;
}

static void charHandler (struct XMLHeap *userData, const XML_Char * s,
                         int len)
{
  strncpy (userData->data[userData->level], s, len);
  userData->data[userData->level][len] = 0;
}

static void fillvalues (struct XMLHeap *userData)
{
  float               f_temp;

  if (!strncasecmp (userData->tags[1], "ADDINFO", 7)) {
    if (!strncasecmp (userData->tags[userData->level], "TOTALDIVES", 10)) {
      userData->diveprofile->DiveNum = atoi (userData->data[userData->level]);
    }
  } else if (!strncasecmp (userData->tags[1], "REPGROUP", 8)) {
    if (!strncasecmp (userData->tags[2], "DIVE", 4)) {
      if (!strncasecmp (userData->tags[3], "DATE", 4)) {
        if (!strncasecmp (userData->tags[userData->level], "DAY", 3)) {
          userData->diveprofile->day = atoi (userData->data[userData->level]);
        } else if (!strncasecmp (userData->tags[userData->level], "MONTH", 5)) {
          userData->diveprofile->month =
              atoi (userData->data[userData->level]);
        } else if (!strncasecmp (userData->tags[userData->level], "YEAR", 4)) {
          userData->diveprofile->year =
              atoi (userData->data[userData->level]);
        }
      } else if (!strncasecmp (userData->tags[3], "TIME", 4)
                 && strlen (userData->tags[3]) == 4) {
        if (!strncasecmp (userData->tags[userData->level], "HOUR", 4)) {
          userData->diveprofile->hour =
              atoi (userData->data[userData->level]);
        } else
            if (!strncasecmp (userData->tags[userData->level], "MINUTE", 6)) {
          userData->diveprofile->minute =
              atoi (userData->data[userData->level]);
        }
      } else
          if (!strncasecmp (userData->tags[userData->level], "TIMEZONE", 8)) {
        strncpy (userData->diveprofile->timezone,
                 userData->data[userData->level],
                 strlen (userData->data[userData->level]) + 1);
      } else
          if (!strncasecmp
              (userData->tags[userData->level], "SURFACEINTERVAL", 15)) {
        if (atoi (userData->data[userData->level]) < 5) {
          userData->diveprofile->SurfaceIntervall = -1;
        } else {
          userData->diveprofile->SurfaceIntervall =
              atoi (userData->data[userData->level]);
        }
      } else
          if (!strncasecmp
              (userData->tags[userData->level], "TEMPERATURE", 11)) {
        userData->diveprofile->temperature =
            atoi (userData->data[userData->level]);
      } else if (!strncasecmp (userData->tags[3], "GASES", 5)) {
        if (!strncasecmp (userData->tags[4], "MIX", 3)) {
          if (!strncasecmp
              (userData->tags[userData->level], "TANKVOLUME", 10)) {
            sscanf (userData->data[userData->level], "%f", &f_temp);
            userData->diveprofile->tanksize = f_temp;
          } else if (!strncasecmp (userData->tags[5], "TANK", 4)) {
            if (!strncasecmp (userData->tags[userData->level], "PDELTA", 6)) {
              userData->diveprofile->pDelta =
                  atol (userData->data[userData->level]) / 100000;
            } else
                if (!strncasecmp
                    (userData->tags[userData->level], "PSTART", 6)) {
              userData->diveprofile->pStart =
                  atoi (userData->data[userData->level]) / 100000;
            } else
                if (!strncasecmp (userData->tags[userData->level], "PEND", 4))
            {
              userData->diveprofile->pEnd =
                  atoi (userData->data[userData->level]) / 100000;
            }
          }
        }
      } else if (!strncasecmp (userData->tags[3], "SAMPLES", 7)) {
        if (!strncasecmp (userData->tags[userData->level], "DELTA", 5)) {
          userData->diveprofile->delta =
              atoi (userData->data[userData->level]);
        } else if (!strncasecmp (userData->tags[userData->level], "D", 1)) {
          sscanf (userData->data[userData->level], "%f", &f_temp);
          userData->diveprofile->profile[userData->diveprofile->numsamples] =
              f_temp;
          userData->diveprofile->avgDepth += f_temp;
          if (userData->diveprofile->
              profile[userData->diveprofile->numsamples] >
              userData->diveprofile->maxDepth) {
            userData->diveprofile->maxDepth =
                userData->diveprofile->profile[userData->diveprofile->
                                               numsamples];
          }
          userData->diveprofile->numsamples++;
        } else if (!strncasecmp (userData->tags[userData->level], "ALARM", 4)) {
          if (!strncasecmp (userData->data[userData->level], "SURFACE", 7)) {
            userData->diveprofile->alarm[userData->diveprofile->numsamples /
                                         3] |= ALARM_SURFACE;
          } else
              if (!strncasecmp (userData->data[userData->level], "DECO", 4)) {
            userData->diveprofile->alarm[userData->diveprofile->numsamples /
                                         3] |= ALARM_DECO;
          } else
              if (!strncasecmp (userData->data[userData->level], "ASCENT", 6))
          {
            userData->diveprofile->alarm[userData->diveprofile->numsamples /
                                         3] |= ALARM_ASCENT;
          }
        }
      }
    }
  }
}


struct DiveProfile *readudcf (char *filename)
{
  static struct DiveProfile diveprofile;
  struct XMLHeap      xmlheap;
  XML_Parser          parser = XML_ParserCreate (NULL);
  int                 done, ret;
  FILE               *fh;
  char                line[1025];

	strcpy(diveprofile.filename, filename);
  xmlheap.diveprofile = &diveprofile;
  xmlheap.level = -1;
  xmlheap.done = 0;
  xmlheap.diveprofile->numsamples = 0;
  xmlheap.diveprofile->maxDepth = 0;
  xmlheap.diveprofile->timezone[0] = 0;

  XML_SetUserData (parser, &xmlheap);
  XML_SetElementHandler (parser, startElementHandler, endElementHandler);
  XML_SetCharacterDataHandler (parser, charHandler);

  fh = fopen (filename, "r");
  if (fh) {
    while (!feof (fh)) {
      ret = fgets (line, sizeof (line), fh);
      done = (ret == EOF) ? 1 : 0;
      // printf("line: %s\n", line);
      if (!XML_Parse (parser, line, strlen (line), done)) {
        fprintf (stderr, "%s at line %d done=%d\n",
                 XML_ErrorString (XML_GetErrorCode (parser)),
                 XML_GetCurrentLineNumber (parser), done);
      }
      line[0] = 0;
    }
    fclose (fh);
  } else {
    printf ("Can't open file %s\n", filename);
  }

  return (&diveprofile);
}
