
/*

  $Id: serial.c,v 1.1 2003/06/08 16:18:07 cramer Exp $
*/

#include <fcntl.h>
#include <sys/termios.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>

#include "include/serial.h"

int openlink (char *device)
{
  struct termios      settings;
  int                 fd;
  int                 modelines = 0;

  fd = open (device, O_RDWR | O_NOCTTY);
  if (fd == -1) {
    fprintf (stderr, "Device %s cannot be opened!\n", device);
    return (fd);
  }

  if (!isatty (fd)) {
    fprintf (stderr, "Device %s is not a terminal-device !\n", device);
    return (-1);
  }

  /*Set inputflags */
  settings.c_iflag = IGNBRK     /* Ignore Input breaks */
      | IGNPAR;                 /* No Parity  */


  /*Set outputflags */
  settings.c_oflag = 0;         /* nothing special */

  /*Set controlflags */
  settings.c_cflag = CS8        /* 8 bits per byte */
      | CREAD                   /* enable receiver */
      | CLOCAL;                 /* Ignore modem status, line is local */

  /*Set local flags */
  settings.c_lflag = 0;         /* nothing special */

  /*Set timeout */
  settings.c_cc[VTIME] = 20;    /* set timeout */

  /*Set minimal bytes to read */
  settings.c_cc[VMIN] = 1;

  /*Set baud rate */
  if (cfsetspeed (&settings, B19200) != 0) {
    fprintf (stderr, "Failed to set baud rate on device %s !\n", device);
    return (-1);
  }

  /*Set parameters defined above to the serial device */
  if (tcsetattr (fd, TCSANOW, &settings) == -1) {
    fprintf (stderr, "Failed to set terminal attributes on device %s !\n",
             device);
    return (-1);
  }

  /*Read io-controll parameters */
  if (ioctl (fd, TIOCMGET, &modelines) == -1) {
    fprintf (stderr, "Error doing ioctl TIOCMGET on device %s!\n", device);
    return (-1);
  }

  /*Set RTS */
  modelines &= ~TIOCM_RTS;
  if (ioctl (fd, TIOCMSET, &modelines) == -1) {
    fprintf (stderr, "Error doing ioctl TIOCMSET on device %s !\n", device);
    return (-1);
  }
  return (fd);
}
